% Lax_Friedrichs scheme for Euler equations
clear; figure(1); clf
%initial_cond		% Definition of initial conditions
problem_specification	

global gamma
gamma = 1.4; gammab = 1/(gamma - 1); gam1 = gamma-1; gamgam = gamma^gamma;
J = 24;	%J = 47;	% J is number of cells
h = 1/J;  dt = lambda*h;
n = floor(tend/dt);	%n = 2;	% n = number of time-steps

% Definition of grid numbering
 
%     x=0    
% grid |---o---|---o---|---o---  ...  --|---o---|
%          1   1   2   2   3           J-1  J  

xcenter = zeros(J,1);
for j = 1:J
  xcenter(j) =(j-1/2)*h;
end

% Initialization of pressure, density and velocity
press = zeros(size(xcenter)); rhoold = press; uold = press;
rhonew = press; mnew = press; totenew = press;
for j = 1:length(xcenter)
  if xcenter(j) < 0.5
    press(j) = pleft;  rhoold(j) = rholeft; uold(j) = uleft;  
  else
    press(j) = pright;  rhoold(j) = rhoright; uold(j) = uright;
  end
end

totenold = rhoold.*(0.5*uold.*uold + gammab*press./rhoold);  % toten = rho*E
totenleft = totenold(1); totenright = totenold(J); mold = rhoold.*uold;
c = sqrt(gamma*press./rhoold); mach = uold./c;	% Sound speed and Mach number

% Declaration of  fluxes
flux1 = zeros(J,1); flux2 = flux1; flux3 = flux1;
flopcount = flops; t0 = cputime;

for i = 1:n
  t = i*dt;
% Computation of Lax_Friedrichs fluxes
  flux1 = mold; flux2 = mold.*uold + press;  flux3 = uold.*(totenold + press);

% Update of state variables
  rhonew(1) = rholeft; rhonew(J) = rhoright;
  mnew(1) = rholeft*uleft; mnew(J) = rhoright*uright;
  totenew(1) = totenleft; totenew(J) = totenright;
  for j = 2:J-1
    rhonew(j)  = 0.5*(rhoold(j-1) + rhoold(j+1)) - ...
                  0.5*lambda*(flux1(j+1) - flux1(j-1));
    mnew(j)    = 0.5*(mold(j-1) +  mold(j+1)) - ...
    		  0.5*lambda*(flux2(j+1) - flux2(j-1));
    totenew(j) = 0.5*(totenold(j-1) + totenold(j+1)) - ...
    		  0.5*lambda*(flux3(j+1) - flux3(j-1));
  end

  uold = mnew./rhonew; press = gam1*(totenew - 0.5*mnew.*uold);
  rhoold = rhonew; totenold = totenew; mold = mnew;
end

% End of time-stepping. Start of postprocessing

flopcount = flops - flopcount; comptime = cputime - t0;
c = sqrt(gamma*press./rhoold); mach = uold./c; 
entropy = log(press./rhoold.^gamma);
subplot(2,3,1)
title('DENSITY'); hold on; plot(xcenter,rhoold,'o');
subplot(2,3,2)
title('VELOCITY'); hold on; plot(xcenter,uold,'o');
subplot(2,3,3)
title('PRESSURE'); hold on; plot(xcenter,press,'o');
subplot(2,3,4)
hold on; title('MACHNUMBER'); plot(xcenter,mach,'o');
subplot(2,3,5)
hold on; title('ENTROPY'); plot(xcenter,entropy,'o');
subplot(2,3,6)
axis('off'); title('Lax-Friedrichs scheme')
text(0,0.9,['lambda = ', num2str(lambda),'  t = ',num2str(n*dt)])
text(0,0.3,['flopcount = ',num2str(flopcount)])
text(0,0.15,['cptime = ',num2str(comptime)])
hold on
Riemann
